=== modified file 'setup.py'
--- setup.py	2007-12-20 23:55:36 +0000
+++ setup.py	2007-12-21 00:10:32 +0000
@@ -219,6 +219,19 @@
     jail_base = query_user(
     """Root directory where user files are stored (on the local file system):
     (eg. "/home/informatics/jails")""")
+    allowed_uid = query_user(
+    """UID of the web server process which will run IVLE.
+Only this user may execute the trampoline. You can configure multiple users
+by manually editing conf.h.
+    (eg. "1002")""")
+
+    # Error handling on input values
+
+    try:
+        allowed_uid = int(allowed_uid)
+    except ValueError:
+        print >>sys.stderr, "Invalid UID (%s)." % allowed_uid
+        return 1
 
     # Write www/conf/conf.py
 
@@ -250,7 +263,7 @@
 # presented with the login screen.
 default_app = "%s"
 """ % (root_dir, ivle_install_dir, jail_base, default_app))
-        
+
         conf.close()
     except IOError, (errno, strerror):
         print "IO error(%s): %s" % (errno, strerror)
@@ -275,7 +288,13 @@
  * jail_base or a subdirectory of jail_base.
  */
 static const char* jail_base = "%s";
-""" % (jail_base))
+
+/* Which user IDs are allowed to run the trampoline.
+ * This list should be limited to the web server user.
+ * (Note that root is an implicit member of this list).
+ */
+static const int allowed_uids[] = { %d };
+""" % (jail_base, allowed_uid))
 
         conf.close()
     except IOError, (errno, strerror):

=== modified file 'trampoline/trampoline.c'
--- trampoline/trampoline.c	2007-12-20 04:49:34 +0000
+++ trampoline/trampoline.c	2007-12-21 00:10:32 +0000
@@ -53,10 +53,38 @@
 
 #define UID_ROOT        0
 
+/* Returns TRUE if the given uid is allowed to execute trampoline.
+ * Only root or the web server should be allowed to execute.
+ * This is determined by the whitelist allowed_uids in conf.h.
+ */
+int uid_allowed(int uid)
+{
+    int i;
+    /* root is always allowed to execute trampoline */
+    if (uid == 0)
+        return 1;
+    /* loop over all allowed_uids */
+    for (i=(sizeof(allowed_uids)/sizeof(*allowed_uids))-1; i>=0; i--)
+    {
+        if (allowed_uids[i] == uid)
+            return 1;
+    }
+    /* default to disallowing */
+    return 0;
+}
+
 int main(int argc, char* const argv[])
 {
     char* jailpath;
     int uid;
+
+    /* Disallow execution from all users but the whitelisted ones, and root */
+    if (!uid_allowed(getuid()))
+    {
+        fprintf(stderr, "only the web server may execute trampoline\n");
+        exit(1);
+    }
+
     /* Args check and usage */
     if (argc < MIN_ARGC)
     {

